@@ -4,6 +4,8 @@ use std::ops::{Mul, Add, Sub, Div, Rem, MulAssign, AddAssign, SubAssign};
 use std::hash::Hash;
 use std::collections::HashMap;
 
+use crate::one::One;
+
 use crate::basic_matrix::basic_matrix::BasicMatrixTrait;
 use crate::basic_matrix::basic_matrix::
 {
@@ -30,54 +32,63 @@ pub enum Operation
 #[derive(Clone)]
 pub struct ExtendedMatrix<T, V>
 {
+    tolerance: V,
     basic_matrix: Box<dyn BasicMatrixTrait<T, V>>
 }
 
 
 impl<T, V> ExtendedMatrix<T, V>
     where T: Copy + Debug + Mul<Output = T> + PartialOrd + Add<Output = T> + Sub<Output = T> +
-             Default + Div<Output = T> + Rem<Output = T> + Eq + Hash + SubAssign + 'static +
-             From<usize> + Into<usize>,
+             Default + Div<Output = T> + Rem<Output = T> + Eq + Hash + SubAssign + One +
+             AddAssign + 'static,
           V: Copy + Debug + PartialEq + Default + AddAssign + MulAssign + Mul<Output = V> +
-             Div<Output = V> + SubAssign + Sub<Output = V> + Add<Output = V> + 'static + Into<f64> +
-             From<f64>,
+             Div<Output = V> + SubAssign + Sub<Output = V> + Add<Output = V> + Into<f64> + One +
+             'static,
 {
     pub fn show_matrix<F>(&self, f: F)
         where F: Fn(&str)
     {
         let shape = self.basic_matrix.get_shape();
-        for i in 0..shape.0.into()
+
+        let mut row = T::default();
+        while row < shape.0
         {
-            let mut row = String::new();
-            for j in 0..shape.1.into()
+            let mut row_str = String::new();
+            let mut column = T::default();
+            while column < shape.1
             {
-                row += &format!("{:?}, ", self.basic_matrix
-                    .read_element_value(T::from(i), T::from(j)).unwrap());
+                row_str += &format!("{:?}, ",
+                    self.basic_matrix.read_element_value(row, column).unwrap());
+                column += T::one();
             }
-            f(&format!("{}", row));
+            f(&format!("{}", row_str));
+            row += T::one();
         }
     }
 
 
-    pub fn create(rows_number: T, columns_number: T, all_elements: Vec<V>, tolerance: f64) -> Self
+    pub fn create(rows_number: T, columns_number: T, all_elements: Vec<V>, tolerance: V) -> Self
     {
         let mut elements_indexes = Vec::new();
         let mut elements_values = Vec::new();
-        let all_elements_length = rows_number * columns_number;
-        for (index, value) in (0..all_elements_length.into()).zip(all_elements)
+
+        let mut index = T::default();
+        for value in all_elements.into_iter()
         {
-            if value.into().abs() > tolerance
+            if value.into().abs() > tolerance.into()
             {
-                elements_indexes.push(T::from(index));
+                elements_indexes.push(index);
                 elements_values.push(value);
             }
+            index += T::one();
         }
+
         let basic_matrix = Box::new(NonSymmetricMatrix
             {
                 columns_number, rows_number, elements_indexes, elements_values
             });
         let basic_matrix = basic_matrix.into_symmetric();
-        ExtendedMatrix { basic_matrix }
+        ExtendedMatrix { tolerance, basic_matrix }
     }
 
 
@@ -99,7 +110,7 @@ impl<T, V> ExtendedMatrix<T, V>
     }
 
 
-    pub fn add_subtract_matrix<'a>(&'a self, other: &'a Self, operation: Operation, tolerance: f64)
+    pub fn add_subtract_matrix<'a>(&'a self, other: &'a Self, operation: Operation)
         -> Result<Self, &'a str>
     {
         let (_, shape) =
@@ -110,14 +121,16 @@ impl<T, V> ExtendedMatrix<T, V>
             other.basic_matrix.extract_all_elements_values();
         let mut elements_indexes = Vec::new();
         let mut elements_values = Vec::new();
-        for index in 0..(shape.0 * shape.1).into()
+
+        let mut index = T::default();
+        while index < shape.0 * shape.1
         {
             let current_lhs_element_value = extract_element_value(
-                    T::from(index) / shape.1, T::from(index) % shape.1,
+                    index / shape.1, index % shape.1,
                     &lhs_all_elements_values
                 );
             let current_rhs_element_value = extract_element_value(
-                    T::from(index) / shape.1, T::from(index) % shape.1,
+                    index / shape.1, index % shape.1,
                     &rhs_all_elements_values
                 );
             let value =
@@ -133,35 +146,37 @@ impl<T, V> ExtendedMatrix<T, V>
                                 applied for add_subtract function!"),
                     }
                 };
-            if value.into().abs() > tolerance
+            if value.into().abs() > self.tolerance.into()
             {
-                elements_indexes.push(T::from(index));
+                elements_indexes.push(index);
                 elements_values.push(value);
             }
+            index += T::one();
         }
+
         let basic_matrix = Box::new(NonSymmetricMatrix
             {
                 rows_number: shape.0, columns_number: shape.1, elements_indexes, elements_values
             });
         let basic_matrix = basic_matrix.into_symmetric();
-        Ok(ExtendedMatrix { basic_matrix })
+        Ok(ExtendedMatrix { tolerance: self.tolerance, basic_matrix })
     }
 
 
-    pub fn add_matrix<'a>(&'a self, other: &'a Self, tolerance: f64) -> Result<Self, &'a str>
+    pub fn add_matrix<'a>(&'a self, other: &'a Self) -> Result<Self, &'a str>
     {
-        self.add_subtract_matrix(other, Operation::Addition, tolerance)
+        self.add_subtract_matrix(other, Operation::Addition)
     }
 
 
-    pub fn subtract_matrix<'a>(&'a self, other: &'a Self, tolerance: f64) -> Result<Self, &'a str>
+    pub fn subtract_matrix<'a>(&'a self, other: &'a Self) -> Result<Self, &'a str>
     {
-        self.add_subtract_matrix(other, Operation::Subtraction, tolerance)
+        self.add_subtract_matrix(other, Operation::Subtraction)
     }
 
 
     pub fn add_sub_matrix(&mut self, other: &Self, self_positions: &[MatrixElementPosition<T>],
-        other_positions: &[MatrixElementPosition<T>], tolerance: f64)
+        other_positions: &[MatrixElementPosition<T>], tolerance: V)
     {
         let lhs_shape = self.basic_matrix.get_shape();
         let lhs_all_elements_values =
@@ -182,29 +197,33 @@ impl<T, V> ExtendedMatrix<T, V>
                     &rhs_all_elements_values
                 );
             let value = current_lhs_element_value + current_rhs_element_value;
-            if value.into().abs() > tolerance
+            if value.into().abs() > tolerance.into()
             {
                 elements_indexes.push(lhs_position.row * lhs_shape.1 + lhs_position.column);
                 elements_values.push(value);
             }
         }
-        for index in 0..(lhs_shape.0 * lhs_shape.1).into()
+
+        let mut index = T::default();
+        while index < lhs_shape.0 * lhs_shape.1
         {
             if let None = self_positions.iter().position(|position|
-                *position == MatrixElementPosition { row: T::from(index) / lhs_shape.1,
-                    column: T::from(index) % lhs_shape.1 })
+                *position == MatrixElementPosition { row: index / lhs_shape.1,
+                    column: index % lhs_shape.1 })
             {
                 let value = extract_element_value(
-                    T::from(index) / lhs_shape.1, T::from(index) % lhs_shape.1,
+                    index / lhs_shape.1, index % lhs_shape.1,
                     &lhs_all_elements_values
                 );
-                if value.into().abs() > tolerance
+                if value.into().abs() > tolerance.into()
                 {
-                    elements_indexes.push(T::from(index));
+                    elements_indexes.push(index);
                     elements_values.push(value);
                 }
             }
+            index += T::one();
         }
+
         let basic_matrix = Box::new(NonSymmetricMatrix
             {
                 rows_number: lhs_shape.0, columns_number: lhs_shape.1, elements_indexes,
@@ -221,7 +240,7 @@ impl<T, V> ExtendedMatrix<T, V>
     }
 
 
-    pub fn multiply_by_matrix<'a>(&'a self, other: &'a Self, tolerance: f64)
+    pub fn multiply_by_matrix<'a>(&'a self, other: &'a Self)
         -> Result<Self, &'a str>
     {
         let (basic_dimension, shape) =
@@ -233,37 +252,45 @@ impl<T, V> ExtendedMatrix<T, V>
             other.basic_matrix.extract_all_elements_values();
         let mut elements_indexes = Vec::new();
         let mut elements_values = Vec::new();
-        for index in 0..(shape.0 * shape.1).into()
+
+        let mut index = T::default();
+        while index < shape.0 * shape.1
         {
             let mut value = V::default();
-            for k in 0..basic_dimension.into()
+
+            let mut k = T::default();
+            while k < basic_dimension
             {
                 let current_lhs_element_value = extract_element_value(
-                        T::from(index) / shape.1, T::from(k),
+                        index / shape.1, k,
                         &lhs_all_elements_values
                     );
                 let current_rhs_element_value = extract_element_value(
-                        T::from(k), T::from(index) % shape.1,
+                        k, index % shape.1,
                         &rhs_all_elements_values
                     );
                 value += current_lhs_element_value * current_rhs_element_value;
+                k += T::one();
             }
-            if value.into().abs() > tolerance
+
+            if value.into().abs() > self.tolerance.into()
             {
                 elements_indexes.push(T::from(index));
                 elements_values.push(value);
             }
+            index += T::one();
         }
+
         let basic_matrix = Box::new(NonSymmetricMatrix
             {
                 rows_number: shape.0, columns_number: shape.1, elements_indexes, elements_values
             });
         let basic_matrix = basic_matrix.into_symmetric();
-        Ok(ExtendedMatrix { basic_matrix })
+        Ok(ExtendedMatrix { tolerance: self.tolerance, basic_matrix })
     }
 
 
-    pub fn naive_gauss_elimination<'a>(&'a self, other: &'a Self, tolerance: f64)
+    pub fn naive_gauss_elimination<'a>(&'a self, other: &'a Self)
         -> Result<Self, &'a str>
     {
         let (basic_dimension, shape) =
@@ -273,208 +300,284 @@ impl<T, V> ExtendedMatrix<T, V>
             self.basic_matrix.extract_all_elements_values();
         let mut rhs_all_elements_values =
             other.basic_matrix.extract_all_elements_values();
-        let mut elements_values = (0..shape.0.into()).map(|_| V::default())
-            .collect::<Vec<V>>();
-        for k in 0..basic_dimension.into() - 1
+
+        let mut elements_values = Vec::new();
+        let mut count = T::default();
+        while count < shape.0
         {
-            for i in (k + 1)..basic_dimension.into()
+            elements_values.push(V::default());
+            count += T::one();
+        }
+
+        let mut k = T::default();
+        while k < basic_dimension - T::one()
+        {
+            let mut i = k + T::one();
+            while i < basic_dimension
             {
-                let current_lhs_element_value = extract_element_value(
-                        T::from(i), T::from(k), &lhs_all_elements_values
-                    );
-                let current_diag_lhs_element_value = extract_element_value(
-                        T::from(k), T::from(k), &lhs_all_elements_values
-                    );
-                let current_coefficient = current_lhs_element_value / current_diag_lhs_element_value;
-                for j in (k + 1)..basic_dimension.into()
+                let current_lhs_element_value = extract_element_value(i, k,
+                    &lhs_all_elements_values);
+
+                let current_diag_lhs_element_value = extract_element_value(k, k,
+                    &lhs_all_elements_values);
+
+                let current_coefficient =
+                    current_lhs_element_value / current_diag_lhs_element_value;
+
+                let mut j = k + T::one();
+                while j < basic_dimension
                 {
-                    let current_lhs_element_value = extract_element_value(
-                            T::from(k), T::from(j), &lhs_all_elements_values
-                        );
+                    let current_lhs_element_value = extract_element_value(k, j,
+                        &lhs_all_elements_values);
+
                     *lhs_all_elements_values
-                        .entry(MatrixElementPosition { row: T::from(i), column: T::from(j) })
+                        .entry(MatrixElementPosition { row: i, column: j })
                         .or_insert(Default::default()) -=
-                        current_coefficient * current_lhs_element_value;
+                            current_coefficient * current_lhs_element_value;
+                    j += T::one();
                 }
-                let current_rhs_element_value = extract_element_value(
-                        T::from(k), T::default(), &rhs_all_elements_values
-                    );
+
+                let current_rhs_element_value = extract_element_value(k,
+                    T::default(), &rhs_all_elements_values);
                 *rhs_all_elements_values
-                    .entry(MatrixElementPosition { row: T::from(i), column: T::default() })
+                    .entry(MatrixElementPosition { row: i, column: T::default() })
                     .or_insert(Default::default()) -=
-                    current_coefficient * current_rhs_element_value;
+                        current_coefficient * current_rhs_element_value;
+                i += T::one();
             }
+            k += T::one();
         }
-        let rhs_element_value = extract_element_value(
-                T::from(basic_dimension.into() - 1), T::default(),
-                &rhs_all_elements_values
-            );
-        let lhs_element_value = extract_element_value(
-                T::from(basic_dimension.into() - 1), T::from(basic_dimension.into() - 1),
-                &lhs_all_elements_values
-            );
-        elements_values[(basic_dimension.into() - 1) as usize] =
-            rhs_element_value / lhs_element_value;
-        for i in (0..basic_dimension.into() - 1).into_iter().rev()
+
+        let rhs_element_value = extract_element_value(basic_dimension - T::one(),
+            T::default(), &rhs_all_elements_values);
+
+        let lhs_element_value = extract_element_value(basic_dimension - T::one(),
+            basic_dimension - T::one(), &lhs_all_elements_values);
+
+        let mut n = 0usize;
+        let mut m = T::default();
+        while m < basic_dimension - T::one()
         {
-            let rhs_element_value = extract_element_value(
-                    T::from(i), T::default(), &rhs_all_elements_values
-                );
-            let mut sum = rhs_element_value;
-            for j in (i + 1)..basic_dimension.into()
-            {
-                let lhs_element_value = extract_element_value(
-                        T::from(i), T::from(j), &lhs_all_elements_values
-                    );
-                sum -= lhs_element_value * elements_values[j as usize];
-            }
-            let lhs_element_value = extract_element_value(
-                    T::from(i), T::from(i), &lhs_all_elements_values
-                );
-            elements_values[i as usize] = sum / lhs_element_value;
+            n += 1usize;
+            m += T::one();
         }
+
+        elements_values[n] = rhs_element_value / lhs_element_value;
+
+        let mut i = basic_dimension - T::one();
+        while i > T::default()
+        {
+            i -= T::one();
+            let rhs_element_value = extract_element_value(i, T::default(),
+                &rhs_all_elements_values);
+
+            let mut sum = rhs_element_value;
+
+            let mut j = i + T::one();
+            while j < basic_dimension
+            {
+                let lhs_element_value = extract_element_value(i, j,
+                    &lhs_all_elements_values);
+
+                let mut n = 0usize;
+                let mut m = T::default();
+                while m < j
+                {
+                    n += 1usize;
+                    m += T::one();
+                }
+
+                sum -= lhs_element_value * elements_values[n];
+                j += T::one();
+            }
+
+            let lhs_element_value = extract_element_value(i, i,
+                &lhs_all_elements_values);
+
+            let mut n = 0usize;
+            let mut m = T::default();
+            while m < i
+            {
+                n += 1usize;
+                m += T::one();
+            }
+            elements_values[n] = sum / lhs_element_value;
+        }
+
         Ok(ExtendedMatrix::create(shape.0, shape.1,
-            elements_values, tolerance))
+            elements_values, self.tolerance))
     }
 
 
-    pub fn lu_decomposition(&self, tolerance: f64) -> Result<(Self, Self), &str>
+    pub fn lu_decomposition(&self) -> Result<(Self, Self), &str>
     {
         let shape = self.basic_matrix.get_shape();
-        if (shape.0 != shape.1) || shape.0 < T::from(2)
+        if (shape.0 != shape.1) || shape.0 < T::one() + T::one()
         {
             return Err("Extended matrix: Matrix could not be decomposed!");
         }
         let mut l_elements_indexes = Vec::new();
         let mut l_elements_values= Vec::new();
-        for i in 0..shape.0.into()
+
+        let mut i = T::default();
+        while i < shape.0
         {
-            l_elements_indexes.push(T::from(i) * shape.1 + T::from(i));
-            l_elements_values.push(V::from(1.0));
+            l_elements_indexes.push(i * shape.1 + i);
+            l_elements_values.push(V::one());
+            i += T::one();
         }
+
         let mut all_elements_values =
             self.basic_matrix.extract_all_elements_values();
         let mut u_elements_indexes = Vec::new();
         let mut u_elements_values= Vec::new();
-        for k in 0..shape.1.into()
+
+        let mut k = T::default();
+        while k < shape.1
         {
-             let current_element_value = extract_element_value(
-                T::default(), T::from(k), &all_elements_values);
-            u_elements_indexes.push(T::from(k));
+            let current_element_value = extract_element_value(T::default(), k,
+                &all_elements_values);
+            u_elements_indexes.push(k);
             u_elements_values.push(current_element_value);
+            k += T::one();
         }
-        let mut row_number = 0;
-        while row_number < shape.0.into() - 1
+
+        let mut row_number = T::default();
+
+        while row_number < shape.0 - T::one()
         {
-            for i in (row_number + 1)..shape.0.into()
+            let mut i = row_number + T::one();
+            while i < shape.0
             {
-                let current_coefficient = extract_element_value(
-                    T::from(i), T::from(row_number), &all_elements_values) /
-                    extract_element_value(T::from(row_number), T::from(row_number),
-                    &all_elements_values);
-                l_elements_indexes.push(T::from(i) * shape.1 + T::from(row_number));
+                let current_coefficient = extract_element_value(i, row_number,
+                    &all_elements_values) / extract_element_value(row_number,
+                    row_number, &all_elements_values);
+
+                l_elements_indexes.push(i * shape.1 + row_number);
                 l_elements_values.push(current_coefficient);
-                for j in 0..shape.1.into()
+
+                let mut j = T::default();
+                while j < shape.1
                 {
-                    let current_element_value = extract_element_value(
-                        T::from(i), T::from(j), &all_elements_values) -
-                        extract_element_value(T::from(row_number), T::from(j),
-                    &all_elements_values) * current_coefficient;
-                    if let Some(position) = u_elements_indexes
-                        .iter()
-                        .position(|index| *index ==  T::from(i) * shape.1 + T::from(j))
+                    let current_element_value = extract_element_value(i, j,
+                        &all_elements_values) - extract_element_value(row_number, j,
+                        &all_elements_values) * current_coefficient;
+
+                    if let Some(position) = u_elements_indexes.iter().position(|index|
+                        *index ==  i * shape.1 + j)
                     {
                         u_elements_values[position] = current_element_value;
                     }
                     else
                     {
-                        u_elements_indexes.push(T::from(i) * shape.1 + T::from(j));
+                        u_elements_indexes.push(i * shape.1 + j);
                         u_elements_values.push(current_element_value);
                     }
                     *all_elements_values
-                        .entry(MatrixElementPosition {
-                            row: T::from(i), column: T::from(j) })
+                        .entry(MatrixElementPosition { row: i, column: j })
                         .or_insert(Default::default()) = current_element_value;
+                    j += T::one();
                 }
+                i += T::one();
             }
-            row_number += 1;
+            row_number += T::one();
         }
-        remove_zero_values(&mut l_elements_indexes, &mut l_elements_values, tolerance);
+
+
+        remove_zero_values(&mut l_elements_indexes, &mut l_elements_values, self.tolerance);
         let l_basic_matrix = Box::new(NonSymmetricMatrix
             {
                 rows_number: shape.0, columns_number: shape.1,
                 elements_indexes: l_elements_indexes, elements_values: l_elements_values
             });
-        let l_matrix = ExtendedMatrix { basic_matrix: l_basic_matrix };
-        remove_zero_values(&mut u_elements_indexes, &mut u_elements_values, tolerance);
+        let l_matrix = ExtendedMatrix { tolerance: self.tolerance, basic_matrix: l_basic_matrix };
+        remove_zero_values(&mut u_elements_indexes, &mut u_elements_values, self.tolerance);
         let u_basic_matrix = Box::new(NonSymmetricMatrix
             {
                 rows_number: shape.0, columns_number: shape.1,
                 elements_indexes: u_elements_indexes, elements_values: u_elements_values
             });
-        let u_matrix = ExtendedMatrix { basic_matrix: u_basic_matrix };
+        let u_matrix = ExtendedMatrix { tolerance: self.tolerance, basic_matrix: u_basic_matrix };
         Ok((l_matrix, u_matrix))
     }
 
 
-    pub fn determinant(&self, tolerance: f64) -> Result<V, &str>
+    pub fn determinant(&self) -> Result<V, &str>
     {
-        let (_, u_matrix) = self.lu_decomposition(tolerance)?;
+        let (_, u_matrix) = self.lu_decomposition()?;
         let u_matrix_elements_values = u_matrix.basic_matrix
             .extract_all_elements_values();
         let shape = u_matrix.basic_matrix.get_shape();
-        let mut determinant = V::from(1.0);
-        for i in 0..shape.0.into()
+        let mut determinant = V::one();
+
+        let mut i = T::default();
+        while i < shape.0
         {
-            let current_diag_element_value = extract_element_value(
-                T::from(i), T::from(i), &u_matrix_elements_values);
+            let current_diag_element_value = extract_element_value(i, i,
+                &u_matrix_elements_values);
             determinant *= current_diag_element_value;
+            i += T::one();
         }
+
         Ok(determinant)
     }
 
 
-    pub fn inverse(&self, tolerance: f64) -> Result<Self, &str>
+    pub fn inverse(&self) -> Result<Self, &str>
     {
         let (l_matrix, u_matrix) =
-            self.lu_decomposition(tolerance)?;
+            self.lu_decomposition()?;
+
         let shape = self.basic_matrix.get_shape();
         let mut inverse_matrix_indexes = Vec::new();
         let mut inverse_matrix_values = Vec::new();
-        for k in 0..shape.1.into()
+
+        let mut k = T::default();
+        while k < shape.1
         {
-            let unit_column_indexes = vec![T::from(k)];
-            let unit_column_values = vec![V::from(1.0)];
+            let unit_column_indexes = vec![k];
+            let unit_column_values = vec![V::one()];
+
             let basic_unit_column = Box::new(NonSymmetricMatrix
             {
-                rows_number: shape.1, columns_number: T::from(1),
+                rows_number: shape.1, columns_number: T::one(),
                 elements_indexes: unit_column_indexes, elements_values: unit_column_values
             });
-            let unit_column = ExtendedMatrix { basic_matrix: basic_unit_column };
+            let unit_column =
+                ExtendedMatrix { tolerance: self.tolerance, basic_matrix: basic_unit_column };
+
             let interim_inverse_column = l_matrix
-                .naive_gauss_elimination(&unit_column, tolerance).unwrap();
+                .naive_gauss_elimination(&unit_column).unwrap();
+
             let inverse_column = u_matrix
-                .naive_gauss_elimination(&interim_inverse_column, tolerance).unwrap();
+                .naive_gauss_elimination(&interim_inverse_column).unwrap();
+
             let all_inverse_column_values =
                 inverse_column.basic_matrix.extract_all_elements_values();
-            for i in 0..shape.0.into()
+
+            let mut i = T::default();
+            while i < shape.0
             {
-                let current_inverse_column_element_value = extract_element_value(
-                T::from(i), T::default(), &all_inverse_column_values);
+                let current_inverse_column_element_value = extract_element_value(i,
+                    T::default(), &all_inverse_column_values);
+
                 if current_inverse_column_element_value != V::default()
                 {
-                    inverse_matrix_indexes.push(T::from(i) * shape.1 + T::from(k));
+                    inverse_matrix_indexes.push(i * shape.1 + k);
                     inverse_matrix_values.push(current_inverse_column_element_value);
                 }
+                i += T::one();
             }
+            k += T::one();
         }
+
         let basic_inverse_matrix = Box::new(NonSymmetricMatrix
             {
                 rows_number: shape.0, columns_number: shape.1,
                 elements_indexes: inverse_matrix_indexes, elements_values: inverse_matrix_values
             });
         let basic_inverse_matrix = basic_inverse_matrix.into_symmetric();
-        Ok(ExtendedMatrix { basic_matrix: basic_inverse_matrix })
+        Ok(ExtendedMatrix { tolerance: self.tolerance, basic_matrix: basic_inverse_matrix })
     }
 
 
